      SUBROUTINE FUELEC (NMATI,  SIN,    NINFI,   SINFI,  NMATO,
     2                   SOUT,   NINFO,  SINFO,   IDSMI,  IDSII,
     3                   IDSMO,  IDSIO,  NTOT,    NSUBS,  IDXSUB,
     4                   ITYPE,  NINT,   INT,     NREAL,  REAL,
     5                   IDS,    NPO,    NBOPST,  NIWORK, IWORK,
     6                   NWORK,  WORK,   NSIZE,   SIZE,   INTSIZ,
     7                   LD   )
C-------------------------------------------------------------------------------
C      Purpose:  This routine is an illustration of using an ASPEN PLUS
C                user-defined unit operation USER2 block to model a fuelcell
C                with H2 and air with two inlets and two outlets.
C                It also illustrates:
C
C                  o The use of component indices for conventional substream,
C
C                  o Calculation of the information streams (work and heat streams),
C
C                  o The use of the IFCMNC function to retrieve physical
C                    property and simulation parameters from the ASPEN
C                    PLUS data structure,
C
C                  o TP and PQ flash using Flsh_Flash subroutine
C
C                  o The SHS_CPACK and DMS_KFORMC ASPEN PLUS utilties and the
C                    AVEMW ASPEN PLUS function.
C
C                  o Write to the GUI Control panel using Terminal Writter Utility
C                    DMS_WRTTRM
C
C     Product
C     Release:   Release 10.1 of ASPEN PLUS
C
C     Created by Steven Qi, 8/5/99
C
C    Supporting
C      Files:    FUELCELL-USER2.BKP
C
C      Usage
C      Notes:    This routine supports two material feed streams
C                (anolyte in and catholyte in), two product streams
C                (anolyte out and catholyte out), one product work stream,
c                and one heat product stream. 
C                
C       Refs:    Aspen Plus 10 User Models Manual
C-------------------------------------------------------------------------------
C
C===============================================================================
C Variable Declarations:
C===============================================================================
C
      IMPLICIT NONE
C
C  Declare integer variables used in demensioning:
C
      INTEGER NMATI, NINFI, NMATO, NINFO,  NTOT,
     +        NSUBS, NINT,  NPO,   NIWORK, NWORK,
     +        NSIZE
C
C  Declare arrays and variables in argument list:
C
      INTEGER IDSMI(2,NMATI), IDSII(2,NINFI),
     +        IDSMO(2,NMATO), IDSIO(2,NINFO),
     +        IDXSUB(NSUBS),  ITYPE(NSUBS), INT(NINT),
     +        IDS(2,3),       NBOPST(6,NPO),
     +        IWORK(NIWORK),  INTSIZ(NSIZE)
      INTEGER NREAL, LD, IMISS, NHSTRY
C
      REAL*8  SIN(NTOT,NMATI),     SINFI(NINFI),
     +        SOUT(NTOT,NMATO),    SINFO(NINFO),
     +        WORK(NWORK), REAL(NREAL), SIZE(NSIZE)
      REAL*8  RMISS 
C
C  Declare Local Variables (variables pertaining to this work)
C
C   Declare variables used in the Flsh_Flash routine and SHS_CPACK utlities.
C   IDXCC (used in the CPACK routines) should be set as large as
C   the number of components. A retention space is created for the
C   flash routine. RETN must be set to 6*NCC+31 as described on
C   the User Models Manual. The length of IRETN should be set to 6. 
C   Alternatively, the retention space available in common STWORK can be used.
C
      INTEGER IRETN(6), IDXCC(50), DMS_KFORMC,
     +        NPKODE, KPHASE,MAXIT, NCP, KODE,
     +        KRESLT, JRES,  LMSG,  LPMSG, ICFLAG
C      
      REAL*8  RETN(151), DUMMY(NTOT), PPUTL_AVEMW, TOL,
     *        TFLOW1, TFLOW2, TFLOW_DUMMY, SPEC1, SPEC2, GUESS
C
C  Other variables used in this work:
C
      INTEGER I
C
      INTEGER IH2, IO2, IN2, IH2O, ICO, ICO2, ICH4
C
      REAL*8  S1, H2,  O2,  N2,  H2O,  CO,  CO2,  CH4,  
     +        O2_REQUIRED, O2_AVAILABLE, H2_FEED, TEMP1, PRES1,
     +        TEMP2, PRES2, ENERGY_AVAILABLE, H_OUTLET
C
      REAL*8  MW_SOUT1, MW_SOUT2, MW_DUMMY, H_IN, H_OUT, MASSFLOW_SIN1,
     +        MASSFLOW_SIN2, CONV, EFF_POWER, EFF_HEAT, POWER_OUTPUT
C
C=======================================================================
C Labelled Commmons: 
C        (SEE Appendix A of the User Models manual)
C=======================================================================
C
#include "dms_ncomp.cmn"
#include "dms_plex.cmn"
#include "dms_maxwrt.cmn"
#include "ppexec_user.cmn"
C
      EQUIVALENCE (RMISS, USER_RUMISS)
      EQUIVALENCE (IMISS, USER_IUMISS)
      EQUIVALENCE (NHSTRY, USER_NHSTRY)
C
C
C===============================================================================
C Begin Executable Code
C===============================================================================
C
C   Locate component indices based on formula name. 
C    These indices correspond to the order that each component is entered
C    in the Components paragraph of the simulation input file or on the GUI
C    Components.Main form. These indices allow the routine to automatically
C    adjust if the component order is changed, or if components are inserted or 
C    deleted from the components paragraph.
C
      IH2O= DMS_KFORMC('H2O')
      ICO=  DMS_KFORMC('CO')
      ICO2= DMS_KFORMC('CO2')
      ICH4= DMS_KFORMC('CH4')
      IH2=  DMS_KFORMC('H2')
      IN2=  DMS_KFORMC('N2')
      IO2=  DMS_KFORMC('O2')
C
C  Define offset to conventional substream, S1 for MIXED.
C 
      S1=IDXSUB(1)-1
C
C  ====== Material Balance =======
C
C  Initialize the outlet stream. Anolyte is stream 1. Catholyte stream 2.
C
      DO I=1, NCOMP_NCC+9
       SOUT(S1+I,1)= SIN(S1+I,1)
       SOUT(S1+I,2)= SIN(S1+I,2)
      END DO
C
C  Calculate the conversion of H2 and O2. If user has a kinetic model, 
C  enter it here. In this file, the H2 conversion is passed as REAL(1)
C  from the user2 block setup in GUI. If no value or a out-of-range value
C  is provided, a default of 80% of H2 conversion is used.
C
C   Fist find total H2 moleflow in the feed
C
      H2_FEED = SIN(S1+IH2,1) + SIN(S1+IH2,2)
C
      IF ((REAL(1).EQ.RMISS).OR.(REAL(1).GT.1).OR.(REAL(1).LT.0)) THEN
	  CONV=0.8
	ELSE
        CONV=REAL(1)
	END IF
C	
      H2O= CONV*H2_FEED
      O2_REQUIRED= H2O/2.
      O2_AVAILABLE= SIN(S1+IO2,1) + SIN(S1+IO2,2)
C
C  Check to see if enough O2 is available. If not, set O2 conversion=1
C
      IF(O2_REQUIRED.GE.O2_AVAILABLE) THEN
          H2O = O2_AVAILABLE*2
       ELSE
          CONTINUE
       END IF
       O2 = - H2O/2
       H2 = - H2O        
C
C
C Set the component flows in the outlet streams.
C
      SOUT(S1+IH2O,1)= SOUT(S1+IH2O,1)
      SOUT(S1+ICO,1) =  SOUT(S1+ICO,1)   
      SOUT(S1+ICO2,1)= SOUT(S1+ICO2,1)  
      SOUT(S1+ICH4,1)= SOUT(S1+ICH4,1)  
      SOUT(S1+IH2,1) =  SOUT(S1+IH2,1) + H2
      SOUT(S1+IN2,1) =  SOUT(S1+IN2,1)
      SOUT(S1+IO2,1) =  SOUT(S1+IO2,1)
C
      SOUT(S1+IH2O,2)= SOUT(S1+IH2O,2) + H2O
      SOUT(S1+ICO,2) =  SOUT(S1+ICO,2)   
      SOUT(S1+ICO2,2)= SOUT(S1+ICO2,2)  
      SOUT(S1+ICH4,2)= SOUT(S1+ICH4,2)  
      SOUT(S1+IH2,2) =  SOUT(S1+IH2,2)
      SOUT(S1+IN2,2) =  SOUT(S1+IN2,2)
      SOUT(S1+IO2,2) =  SOUT(S1+IO2,2) + O2
C
C     Find total flow and average molecular weight of the two outlet streams:
C
      CALL SHS_CPACK (SOUT(S1+1,1), NCP, IDXCC, RETN, TFLOW1)
      MW_SOUT1=PPUTL_AVEMW(NCP,IDXCC,RETN)
C
      CALL SHS_CPACK (SOUT(S1+1,2), NCP, IDXCC, RETN, TFLOW2)
      MW_SOUT2=PPUTL_AVEMW(NCP,IDXCC,RETN)
C
C  The total flow and molecular weight in the outlet stream vector are given by:
C	
      SOUT(S1+NCOMP_NCC+1,1)= TFLOW1
      SOUT(S1+NCOMP_NCC+1,2)= TFLOW2
      SOUT(S1+NCOMP_NCC+9,1)= MW_SOUT1
      SOUT(S1+NCOMP_NCC+9,2)= MW_SOUT2
C
C ======  Enthalpy Balance  =======
C
C     Mass flow of the feeds:
C
       MASSFLOW_SIN1=SIN(S1+NCOMP_NCC+1,1)*SIN(S1+NCOMP_NCC+9,1)
       MASSFLOW_SIN2=SIN(S1+NCOMP_NCC+1,2)*SIN(S1+NCOMP_NCC+9,2)
C
C     Find the total enthalpy for the two feed streams:
C  
       H_IN= MASSFLOW_SIN1*SIN(S1+NCOMP_NCC+4,1)
     +      +MASSFLOW_SIN2*SIN(S1+NCOMP_NCC+4,2)
C     
C  Perform a TP flash (KODE=2) to determine the outlet enthalpy at the same temp
C  as the inlet
C
      MAXIT=30
      TOL=1D-6
C
C     Slways calculate results
      KRESLT= 1
C
C     Disable restart
      JRES=0
C
C  Specify 2 phases using NPKODE variable.
      NPKODE=2
C  
C  Set the temp and pressure for the outlet streams and then
C  perform a TP flash, assuming no pressure drop along the cell.
C
      TEMP1=SIN(S1+NCOMP_NCC+2,1)
      TEMP2=SIN(S1+NCOMP_NCC+2,2)
      PRES1=SIN(S1+NCOMP_NCC+3,1)
      PRES2=SIN(S1+NCOMP_NCC+3,2)
      SPEC1=TEMP1
      SPEC2=PRES1
      KODE= 2
C
      CALL FLSH_FLASH (SOUT(S1+1,1),NSUBS, IDXSUB, ITYPE, NBOPST,
     +                 KODE,        NPKODE,KPHASE, MAXIT, TOL,
     +                 SPEC1,       SPEC2, GUESS,  LMSG,  LPMSG,
     +                 JRES,        KRESLT,RETN,   IRETN, ICFLAG)
C
      SPEC1=TEMP2
      SPEC2=PRES2
C
      CALL FLSH_FLASH (SOUT(S1+1,2),NSUBS, IDXSUB, ITYPE, NBOPST,
     +                 KODE,        NPKODE,KPHASE, MAXIT, TOL,
     +                 SPEC1,       SPEC2, GUESS,  LMSG,  LPMSG,
     +                 JRES,        KRESLT,RETN,   IRETN, ICFLAG)

C
C  The isothermal enthalpy in the outlet is:
C
       H_OUT =  SOUT(S1+NCOMP_NCC+4,1)*TFLOW1*MW_SOUT1
     +         +SOUT(S1+NCOMP_NCC+4,2)*TFLOW2*MW_SOUT2
C
C  The total available energy is the difference in the isothermal
C  enthalpy between inlet and outlet streams:
C
       ENERGY_AVAILABLE = H_IN-H_OUT
C
C  Only a percentage of this
C  available energy can be converted to power output. EFF_POWER is defined
C  as the fraction and is either calculated by a model or set by the user 
C  and passed as a fraction via REAL(2) from user2 setup in GUI. If no value or an 
C  out-of-range value (such as a value greater than one or less than zero) 
C  is provided by the user, a default of 70% is used.
C
C  The cell usually has some cooling system to remove the heat. Here again, the 
C  percentage of the available power that has been converted to heat that is
C  removed is defined as HEATLOSS_EFF and is passed as REAL(3). If no value or
C  an out-of-range value is given, a default 25% is assumed, i.e., 25% is removed
C  as heat. The remainder (default 5%) stays with the outlet streams, which 
C  will cause some temperature rise.
C 
      IF((REAL(2).EQ.RMISS).OR.(REAL(2).GT.1).OR.(REAL(2).LT.0)) THEN 
	 EFF_POWER=0.7
	ELSE
       EFF_POWER=REAL(2)
	END IF
C
      IF((REAL(3).EQ.RMISS).OR.(REAL(3).GT.1).OR.(REAL(3).LT.0)) THEN
        EFF_HEAT=0.25
	ELSE
        EFF_HEAT=REAL(3)
	END IF
C
C  Power output:
C
        POWER_OUTPUT = ENERGY_AVAILABLE*EFF_POWER
C    
C  Work stream must be negative for positive power output
C  based on Aspen Plus sign convention (SINFO(2) is work stream):
C 
        SINFO(2)= -POWER_OUTPUT
C
C  Heat loss (SINFO(1) is heat stream):
C
        SINFO(1)= ENERGY_AVAILABLE*EFF_HEAT
C
C
C ====== Determine outlet temperature =======
C
C  Assuming a perfect heat transfer between the anode and cathode compartment,
C  the outlet temp for both outlet streams should be near the same. 
C  To find the outlet temperature, first define a a dummy stream representing the 
C  mixture of the two outlet streams:
C
      DO I=1, NCOMP_NCC+1
       DUMMY(S1+I)= SOUT(S1+I,1)+SOUT(S1+I,2)
      END DO
C
C  Find total flow and average molecular weight of the dummy stream
C
      CALL SHS_CPACK (DUMMY(S1+1), NCP, IDXCC, RETN, TFLOW_DUMMY)
      MW_DUMMY=PPUTL_AVEMW(NCP,IDXCC,RETN)
C
C  Total enthalpy in the mixture outlet stream:
C
      H_OUTLET = H_IN - ENERGY_AVAILABLE*(EFF_POWER+EFF_HEAT)
C
C  Find the enthalpy of the dummy stream by enthalpy balance:
C
      DUMMY(S1+NCOMP_NCC+4)=H_OUTLET/TFLOW_DUMMY/MW_DUMMY
C
C  Perform a PQ flash (KODE=1) to determine the temperature for the mixture. SPEC1 is 
C  pressure, and SPEC2 is Q. Assume no pressure drop. Use their average if 
C  two inlets are at different pressure:
C
      KODE=1
      NPKODE=2
      SPEC1=(PRES1+PRES2)/2
      SPEC2=0
      GUESS=(SIN(S1+NCOMP_NCC+2,1)+SIN(S1+NCOMP_NCC+2,2))/2
C
      CALL FLSH_FLASH (DUMMY(S1+1), NSUBS, IDXSUB, ITYPE, NBOPST,
     +                 KODE,        NPKODE,KPHASE, MAXIT, TOL,
     +                 SPEC1,       SPEC2, GUESS,  LMSG,  LPMSG,
     +                 JRES,        KRESLT,RETN,   IRETN, ICFLAG)
C
C  Set both outlet stream temp equal to the temp of the dummy stream.
C  This is the initial guess for the outlet temperature.
C	
	SOUT(S1+NCOMP_NCC+2,1) = DUMMY(S1+NCOMP_NCC+2)
	SOUT(S1+NCOMP_NCC+2,2) = DUMMY(S1+NCOMP_NCC+2)
C
C  Now perform TP flash (KODE=2) on one outlet stream to determine outlet conditions.
C  SPEC1 is temp, and SPEC2 is pressure.
C  Assume no pressure drop through the cell. 
C
C
  50  SPEC1=(SOUT(S1+NCOMP_NCC+2,1)+SOUT(S1+NCOMP_NCC+2,2))/2
      SPEC2=PRES1
      KODE=2
      NPKODE=2
C
      CALL FLSH_FLASH (SOUT(S1+1,1),NSUBS, IDXSUB, ITYPE, NBOPST,
     +                 KODE,        NPKODE,KPHASE, MAXIT, TOL,
     +                 SPEC1,       SPEC2, GUESS,  LMSG,  LPMSG,
     +                 JRES,        KRESLT,RETN,   IRETN, ICFLAG)
C
C   For the second outlet stream, perform a PQ flash (KODE=1). SPEC1 is pressure
C   SPEC2 is Q. The enthalpy for the second stream will be the enthalpy for the
C   total mixture (two streams combined) minus the enthalpy in the first outlet stream
C
C
      SOUT(S1+NCOMP_NCC+4,2)=(H_OUTLET-SOUT(S1+NCOMP_NCC+4,1)
     +                       *TFLOW1*MW_SOUT1)/TFLOW2/MW_SOUT2
      KODE=1
      SPEC1=PRES2
      SPEC2=0
      GUESS=SOUT(S1+NCOMP_NCC+2,1)
C
      CALL FLSH_FLASH (SOUT(S1+1,2),NSUBS, IDXSUB, ITYPE, NBOPST,
     +                 KODE,        NPKODE,KPHASE, MAXIT, TOL,
     +                 SPEC1,       SPEC2, GUESS,  LMSG,  LPMSG,
     +                 JRES,        KRESLT,RETN,   IRETN, ICFLAG) 
C
C  Check to see if the temperature for both outlets is roughly the same
C  (within 1 K from each other). If not, use the average of the two temperature
C  and repeat the above flashes.
C 
      IF(ABS(SOUT(S1+NCOMP_NCC+2,1)-SOUT(S1+NCOMP_NCC+2,2)).GT.1)
     +     GO TO 50     
C
      WRITE (MAXWRT_MAXBUF,100) CONV, EFF_HEAT, EFF_POWER,
     +                          SINFO(1), -SINFO(2)
      CALL DMS_WRTTRM(7)
C
100   FORMAT(/, 'CALCULATED FROM USER2 SUBROUTINE:',/,
     +          'CONVERSION OF H2                  = ', F15.2,/,
     +          'FRAC OF ENERGY REMOVED AS HEAT    = ', F15.2,/,
     +          'FRAC OF ENERGY CONVERTED TO POWER = ', F15.2,/,
     +          'HEAT REMOVED FROM CELL            = ', F15.2, ' J/S',/,
     +          'POWER OUTPUT FROM CELL            = ', F15.2, ' J/S')
C
999     CONTINUE
      RETURN
      END


