      SUBROUTINE FUELEC (NMATI,  SIN,    NINFI,   SINFI,  NMATO,
     2                   SOUT,   NINFO,  SINFO,   IDSMI,  IDSII,
     3                   IDSMO,  IDSIO,  NTOT,    NSUBS,  IDXSUB,
     4                   ITYPE,  NINT,   INT,     NREAL,  REAL,
     5                   IDS,    NPO,    NBOPST,  NIWORK, IWORK,
     6                   NWORK,  WORK,   NSIZE,   SIZE,   INTSIZ,
     7                   LD   )
C-------------------------------------------------------------------------------
C      Purpose:  This routine is an illustration of using an ASPEN PLUS
C                user-defined unit operation USER2 block to model a fuelcell
C                with H2 and air.
C                It also illustrates:
C
C                  o The use of component indices for conventional substream,
C
C                  o Calculate the information stream (work or heat streams),
C
C                  o The use of the IFCMNC function to retrieve physical
C                    property and simulation parameters from the ASPEN
C                    PLUS data structure,
C
C                  o The FLASH physical property routine for a TP flash,
C
C                  o The SHS_CPACK and DMS_KFORMC ASPEN PLUS utilties and the
C                    AVEMW ASPEN PLUS function.
C
C                  o Write to the GUI Control panel using Terminal Writter Utility
C                    DMS_WRTTRM
C
C     Product
C     Release:   Release 10.1 of ASPEN PLUS
C
C     Created by Steven Qi, 8/99
C
C    Supporting
C      Files:    FUELCELL-USER2.BKP
C
C      Usage
C      Notes:    This routine supports two material feed streams,
C                one material product stream, a single product work stream,
c                and a single heat product stream. 
C                
C       Refs:    Aspen Plus 10 User Models Manual
C-------------------------------------------------------------------------------
C
C===============================================================================
C Variable Declarations:
C===============================================================================
C
      IMPLICIT NONE
C
C  Declare Variables Used in Demensioning:
C
      INTEGER NMATI, NINFI, NMATO, NINFO,  NTOT,
     +        NSUBS, NINT,  NPO,   NIWORK, NWORK,
     +        NSIZE
C
C  Declare Arguments (arrays and variables in argument list)
C
      INTEGER IDSMI(2,NMATI), IDSII(2,NINFI),
     +        IDSMO(2,NMATO), IDSIO(2,NINFO),
     +        IDXSUB(NSUBS),  ITYPE(NSUBS), INT(NINT),
     +        IDS(2,3),       NBOPST(6,NPO),
     +        IWORK(NIWORK),  INTSIZ(NSIZE)
      INTEGER NREAL, LD, IMISS
C
      REAL*8  SIN(NTOT,NMATI),     SINFI(NINFI),
     +        SOUT(NTOT,NMATO),    SINFO(NINFO),
     +        WORK(NWORK), REAL(NREAL), SIZE(NSIZE)
      REAL*8  RMISS 
C
C  Declare Local Variables (variables pertaining to this work)
C
C   Declare variables used in the Flsh_Flash routine and SHS_CPACK utlities.
C   IDXCC (used in the CPACK routines) should be set as large as
C   the number of components. A retention space is created for the
C   flash routine. RETN must be set to 6*NCC+31 as described on
C   the User Models Manual. The length of IRETN should be set to 6. 
C   Alternatively, the retention space available in common STWORK can be used.
C
      INTEGER IRETN(6), IDXCC(50), DMS_KFORMC,
     +        NPKODE, KPHASE,MAXIT, NCP,   KODE,
     +        KRESLT, JRES,  LMSG,  LPMSG, ICFLAG
C      
      REAL*8  RETN(151), PPUTL_AVEMW, 
     *        TOL, TFLOW, SPEC1, SPEC2, GUESS
C
C  Other variables used in this work:
C
      INTEGER NTERM, NHSTRY, I
C
      INTEGER IH2, IO2, IN2, IH2O, ICO, ICO2, ICH4
C
      REAL*8  S1, H2,  O2,  N2,  H2O,  CO,  CO2,  CH4,  
     +        O2_REQUIRED, O2_AVAILABLE, H2_FEED, PRES, TEMP
C
      REAL*8  MW_SIN1, MW_SIN2, MW_OUT, MASSFLOW_SIN1, MASSFLOW_SIN2, 
     +        H_IN, H_OUT, POWER_OUTPUT
C
C=======================================================================
C Labelled Commmons: 
C        (SEE Appendix A of the User Models manual)
C=======================================================================
C
#include "dms_ncomp.cmn"
#include "dms_plex.cmn"
#include "dms_maxwrt.cmn"
#include "ppexec_user.cmn"
C
      EQUIVALENCE (RMISS, USER_RUMISS)
      EQUIVALENCE (IMISS, USER_IUMISS)
      EQUIVALENCE (NHSTRY, USER_NHSTRY)
C
C
C===============================================================================
C Begin Executable Code
C===============================================================================
C
C   Locate component indices based on formula name. 
C    These indices correspond to the order that each component is entered
C    in the Components paragraph of the simulation input file or on the GUI
C    Components.Main form. These indices allow the routine to automatically
C    adjust if the component order is changed, or if components are inserted or 
C    deleted from the components paragraph.
C
      IH2O= DMS_KFORMC('H2O')
      ICO= DMS_KFORMC('CO')
      ICO2= DMS_KFORMC('CO2')
      ICH4= DMS_KFORMC('CH4')
      IH2= DMS_KFORMC('H2')
      IN2= DMS_KFORMC('N2')
      IO2= DMS_KFORMC('O2')
C
C  Define offset to conventional substream, S1 for MIXED.
C 
      S1=IDXSUB(1)-1
C
C
C  Initialize the outlet stream
C
      DO I=1, NCOMP_NCC
       SOUT(S1+I,1)= SIN(S1+I,1)+SIN(S1+I,2)
      END DO
C
C
C  Calculate the conversion of H2 and O2. If user has a kinetic model, 
C  enter it here. In this file, a 80% of H2 conversion is simply assumed
C
C   Fist find total H2 moleflow in the feed
C
      H2_FEED = SIN(S1+IH2,1) + SIN(S1+IH2,2)
C
C   80% Conversion:
C
      H2O= 0.8*H2_FEED
      O2_REQUIRED= H2O/2.
      O2_AVAILABLE= SIN(S1+IO2,1) + SIN(S1+IO2,2)
C
C  Check to see if enough O2 is available. If not, set O2 conversion=1
C
      IF(O2_REQUIRED.GE.O2_AVAILABLE) THEN
          H2O = O2_AVAILABLE*2
       ELSE
          CONTINUE
       END IF
       O2 = - H2O/2
       H2 = - H2O        
C
C
C Set the component flows in the outlet stream.
C
      SOUT(S1+IH2O,1)= SOUT(S1+IH2O,1)  + H2O
      SOUT(S1+ICO,1)=  SOUT(S1+ICO,1)   
      SOUT(S1+ICO2,1)= SOUT(S1+ICO2,1)  
      SOUT(S1+ICH4,1)= SOUT(S1+ICH4,1)  
      SOUT(S1+IH2,1)=  SOUT(S1+IH2,1) + H2
      SOUT(S1+IN2,1)=  SOUT(S1+IN2,1)
      SOUT(S1+IO2,1)=  SOUT(S1+IO2,1) + O2
C
C
C Enthalpy Calculation
C
C     Molecular Weight for the feed streams:
C 
       MW_SIN1 = SIN(S1+NCOMP_NCC+9,1)
       MW_SIN2 = SIN(S1+NCOMP_NCC+9,2)
C
C     Total Mole flow for the feeds:
C
       MASSFLOW_SIN1=SIN(S1+NCOMP_NCC+1,1)*MW_SIN1
       MASSFLOW_SIN2=SIN(S1+NCOMP_NCC+1,2)*MW_SIN2
C
C     Find the total enthalpy for the two feed streams:
C  
       H_IN= MASSFLOW_SIN1*SIN(S1+NCOMP_NCC+4,1)
     +      +MASSFLOW_SIN2*SIN(S1+NCOMP_NCC+4,2)
C
C     Find total flow and average molecular weight of product
C
      CALL SHS_CPACK (SOUT(S1+1,1), NCP, IDXCC, RETN, TFLOW)
      MW_OUT=PPUTL_AVEMW(NCP,IDXCC,RETN)
C     
C  Perform a TP flash (KODE=2) to determine the outlet enthalpy at the same temp
C  as the inlet
C
C
      MAXIT=30
      TOL=1D-4
C
C     Slways calculate results
      KRESLT= 1
C
C     Disable restart
      JRES=0
C
C  Specify 2 phases using NPKODE variable.  
      KODE= 2
      NPKODE=2
C  
C  If the two feeds are at different temperature and presure, use the average for flsh_flash
C
      TEMP=(SIN(S1+NCOMP_NCC+2,1)+SIN(S1+NCOMP_NCC+2,2))/2
      PRES=(SIN(S1+NCOMP_NCC+3,1)+SIN(S1+NCOMP_NCC+3,2))/2
      SPEC1=TEMP
      SPEC2=PRES
      SOUT(S1+NCOMP_NCC+1,1)= TFLOW
C
      CALL FLSH_FLASH (SOUT(S1+1,1),NSUBS, IDXSUB, ITYPE, NBOPST,
     +                 KODE,        NPKODE,KPHASE, MAXIT, TOL,
     +                 SPEC1,       SPEC2, GUESS,  LMSG,  LPMSG,
     +                 JRES,        KRESLT,RETN,   IRETN, ICFLAG)
C
C  The isothermal enthalpy in the outlet is:
C
       H_OUT = SOUT(S1+NCOMP_NCC+4,1)*TFLOW*MW_OUT
C
C  Assume 90% of the enthalpy difference is converted to power and 5% to heat loss
C  SINFO(1) is heat stream, SINFO(2) is work stream:
C
        SINFO(1)= (H_IN-H_OUT)*0.05
        POWER_OUTPUT = (H_IN-H_OUT)*0.9
C    
C  Work stream must be converted to watts and must be negative for positive power output
C  based on Aspen Plus sign convention. 
C      
         SINFO(2)= -POWER_OUTPUT*0.29307107
C 
         SINFO(2)= -POWER_OUTPUT
C 
C  Enthlapy Balance
C
      SOUT(S1+NCOMP_NCC+4,1)=(H_IN -POWER_OUTPUT-SINFO(1))/
     +                        (TFLOW*MW_OUT)
C
      WRITE (MAXWRT_MAXBUF,100) SINFO(1), -SINFO(2)
      CALL DMS_WRTTRM(5)
100   FORMAT(/, 'CALCULATED FROM USER2 SUBROUTINE:',/,
     +          'HEAT LOSS FROM CELL    = ', F15.2, ' J/S',/,
     +          'POWER OUTPUT FROM CELL = ', F15.2, ' J/S')
C
999     CONTINUE
      RETURN
      END
